# Generated by roxygen2: do not edit by hand

S3method(coef,enriched_glm)
S3method(coef,enriched_lm)
S3method(enrich,"link-glm")
S3method(enrich,betareg)
S3method(enrich,family)
S3method(enrich,glm)
S3method(enrich,lm)
S3method(get_auxiliary_functions,betareg)
S3method(get_auxiliary_functions,glm)
S3method(get_auxiliary_functions,lm)
S3method(get_bias_function,betareg)
S3method(get_bias_function,glm)
S3method(get_bias_function,lm)
S3method(get_dmodel_function,glm)
S3method(get_enrichment_options,"link-glm")
S3method(get_enrichment_options,betareg)
S3method(get_enrichment_options,family)
S3method(get_enrichment_options,glm)
S3method(get_enrichment_options,lm)
S3method(get_information_function,betareg)
S3method(get_information_function,glm)
S3method(get_information_function,lm)
S3method(get_pmodel_function,glm)
S3method(get_qmodel_function,glm)
S3method(get_score_function,betareg)
S3method(get_score_function,glm)
S3method(get_score_function,lm)
S3method(get_simulate_function,betareg)
S3method(get_simulate_function,glm)
S3method(get_simulate_function,lm)
S3method(print,enrichment_options)
export(create_enrichwith_skeleton)
export(enrich)
export(enriched_glm)
export(get_auxiliary_functions)
export(get_bias_function)
export(get_dmodel_function)
export(get_enrichment_options)
export(get_information_function)
export(get_pmodel_function)
export(get_qmodel_function)
export(get_score_function)
export(get_simulate_function)
import(stats)
